program testkey;
{ test ncurses for f key and character input  }
{ compile for GPC: gpc --automake --ignore-function-results -lncurses testkey.pas -o testkey
}

uses ncurses;

var
  c1, c2: chtype; { chtype is the ncurses "char" type (actually integer) returned by getch etc }
  i, line, col: integer;
  linecount, colcount: int32; { MedInt; } { signed 32-bit }

function IsFKey(c: chtype): boolean;
{ return true iff c is a function key, }
begin
  IsFKey :=  ( c >=  KEY_MIN ) AND ( c<= KEY_MAX );
end { IsFKey };

function UserKeyName(s: pchar): pchar;
  { return a user-friendly key name, e.g. "KEY_F(2)"  => "F2"  }
var sx: string[120]; ix: integer;
begin
  sx := CString2String(s);
  ix :=  index(sx,'KEY_');
  if (ix=1) and (length(sx)>4) then sx :=  substr(sx,5);
  ix := index(sx,'(');
  if (ix>1) and (ix+1<length(sx)) then 
    begin  
      sx := sx[1..ix-1] + sx[ix+1..length(sx)];  { trim the "(" }
      if sx[length(sx)] = ')' then sx :=substr(sx,1,length(sx)-1);
    end;
  UserKeyName := sx;
end { UserKeyName };

procedure AddInt(cc: integer);
{ puts a three-digit integer in the output buffer }
begin
        addch(ord('0') + cc div 100);
        cc :=  cc mod 100;
        addch(ord('0') + (cc div 10) );
        cc := cc mod 10;
        addch(ord('0') + (cc ) );
end { AddInt };

begin
  writeln('About to set up ncurses ..'); { if setup fails, this stays on screen }
  writeln('terminal name is said to be ', CString2String(termname));
  writeln('type RETURN, and you will enter RAW mode, exit only as instructed (no ^C)');
  readln;
{ ncurses initialization }
   initscr;
   clear; { initscr also clears in most, not all, implementations }
   noecho;
   { cbreak; }    { No line buffering, pass on as typed including erase, but process ^C and ^Z }
   raw;   { raw additionally does not interpret interrupt, quit, suspend, and flow control  characters }
   keypad(stdscr, true);  { enable ncurses' recognition of F keys }
   
   move(1,1);
   getbegyx(stdscr, linecount, colcount);
   addstr('begin line, begin col = ');AddInt(linecount); addstr(', '); AddInt(colcount); refresh;
   getmaxyx(stdscr, linecount, colcount);
   move(0,0);
   addstr('linecount, colcount = ');  AddInt(linecount); addstr(', '); AddInt(colcount); refresh;
   c2 := getch; { pause for user }
   clear;
   line := -1;  col:= 0; 
   for c1 := KEY_MIN to KEY_MAX do 
     begin
       line := line + 1;
       if line>=linecount then 
            begin line:=0; col:=col+20;
               if col+10 >= colcount then 
                  begin refresh; c2:=getch; col:=0; clear;  end;
             end;
       move( line, col);   AddInt(c1);
       addstr(': ');  addstr(keyname(c1)); 
       if has_key(c1) then addstr('*');
       refresh;
     end;
   c1 := getch;
   clear;
   move(0,0); { line, col, both zero-origin }
   addstr('type some characters and F keys, "q" to quit: ');
   refresh;
   repeat
     c1 := getch;
     move(1,0); clrtoeol;  addstr('Is F Key=');
     if IsFKey(c1) then addstr('True  ') else addstr('False ');
     addstr('label="');  
     addstr(UserKeyName(keyname(c1)));
     addstr('"   value='); AddInt(c1);
     addstr('   chr="'); addch(c1); addstr('" ');
    refresh; 
   until ord('q') = c1;
   { last info should be left on screen }
   endwin; 
end.

